<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Functions.inc.php                                              05/22/2003
 * This script contains a set of functions that will make it easier to get
 * information from the Virtual Advanced configuration and data files. It also
 * contains basic functions to aid you when creating new scripts.
 *
 * This script should always be included first in any script that will use any
 * VADV-PHP functions.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2003 Steve Winn. All Rights Reserved.
 *
 * The source code contains a fair amount of comments, which may aid anyone
 * attempting to follow the program or to modify it.
 *
 * For licensing and copyright information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');                    // Get include directory.
require_once($includeDir . 'common.inc.php');       // Include common settings.
require_once($includeDir . 'va-syspaths.inc.php');  // Include the BBS's paths.
require_once($includeDir . 'va-loginauth.inc.php'); // Login authentication.

$VA_USERS = UpdateWhosOnline();                     // Update the WEBCALLER.LST file.


/**
* Va-Functions.inc.php :: ReadCfgFile()
*
* This function will read the entire file specified into one variable.
*
*/
function ReadCfgFile($file) {
    if (!is_string($file)) trigger_error(VA_Error(1));
    
    // Check to see if the file exists, if not return FALSE.
    if (!is_file($file)) return '';         // If file is missing, then it is blank.

    $length = filesize($file);              // Determine the length of the file.

    $fp = fopen($file, 'r');                // Open the file.
    $fileString = fread($fp, $length);      // Read the entire length into a variable.
    fclose($fp);                            // Close the file.

    return $fileString;                     // Return the file string.
}


/**
* Va-Functions.inc.php :: WriteCfgFile()
*
* This function will create a new configuration file based on the file string
* passed.
*
*/
function WriteCfgFile($file, $fileString) {
    if (!is_string($file))       trigger_error(VA_Error(1));
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    
    $length = strlen($fileString);          // Determine the length of the file string.
    
    $fp = fopen($file, 'w');                // Open the file.
    flock($fp, 2);                          // Lock the file.

    $status = fwrite($fp, $fileString, $length);    // Write the new file.

    flock($fp, 3);                          // Unlock the file.
    fclose($fp);                            // Close the file.
    
    return $status;                         // Return the write status.
}


/**
* Va-Functions.inc.php :: GetString()
*
* This function will take the given file string and extract the string value
* by using its position and length.
*
*/
function GetString($fileString, $position, $length) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($position))) trigger_error(VA_Error(1));
    if ((!is_int($length))       && (!is_string($length)))   trigger_error(VA_Error(1));

    // Check to see if the variable was passed, if not then return nothing.
    if ($fileString == '') return '';

    // Check to see if the position is larger than the string length.
    if ($position > strlen($fileString)) return '';

    $position--;                            // Reduce the position by one.
    $string = trim(substr($fileString, $position, $length));

    return strval($string);                 // Return the string.
}


/**
* Va-Functions.inc.php :: GetChar()
*
* This function will take the given file string and extract the character value
* by using its position.
*
*/
function GetChar($fileString, $position) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($position))) trigger_error(VA_Error(1));

    // Check to see if the variable was passed, if not then return nothing.
    if ($fileString == '') return '';

    // Check to see if the position is larger than the string length.
    if ($position > strlen($fileString)) return 0;

    $position--;                            // Reduce the position by one.
    // Get the first ascii value.
    $a = ord(substr($fileString, $position, 1));

    return intval($a);                      // Return the integer.
}


/**
* Va-Functions.inc.php :: GetInteger()
*
* This function will take the given file string and extract the integer value
* by using its position.
*
*/
function GetInteger($fileString, $position) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($position))) trigger_error(VA_Error(1));

    // Check to see if the variable was passed, if not then return nothing.
    if ($fileString == '') return '';

    // Check to see if the position is larger than the string length.
    if ($position > strlen($fileString)) return 0;

    $position--;                            // Reduce the position by one.
    // Get the first ascii value.
    $a = ord(substr($fileString, $position, 1));

    $position++;                            // Move to the next position.
    // Get the second ascii value.
    $b = 256 * ord(substr($fileString, $position, 1));

    return intval($a + $b);                 // Return the integer.
}


/**
* Va-Functions.inc.php :: GetLongInt()
*
* This function will take the given file string and extract the long integer
* value by using its position.
*
*/
function GetLongInt($fileString, $position) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($position))) trigger_error(VA_Error(1));

    // Check to see if the variable was passed, if not then return nothing.
    if ($fileString == '') return '';
    
    // Check to see if the position is larger than the string length.
    if ($position > strlen($fileString)) return 0;
    
    $position--;                            // Reduce the position by one.
    // Get the first ascii value.
    $a = ord(substr($fileString, $position, 1));
    
    $position++;                            // Move to the next position.
    // Get the second ascii value.
    $b = 256 * ord(substr($fileString, $position, 1));

    $position++;                            // Move to the next position.
    // Get the third ascii value.
    $c = 65536 * ord(substr($fileString, $position, 1));
    
    $position++;                            // Move to the next position.
    // Get the fourth ascii value.
    $d = 16777216 * ord(substr($fileString, $position, 1));
    
    return intval($a + $b + $c + $d);       // Return the long integer.
}


/**
* Va-Functions.inc.php :: GetSingle()
*
* This function will take the given file string and extract the single precison
* value by using its position.
*
*/
function GetSingle($fileString, $position) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($position))) trigger_error(VA_Error(1));

// ** It has not been determined how to extract this value! **
}


/**
* Va-Functions.inc.php :: PutString()
*
* This function will put the passed string into the file string at the given
* position.
*
*/
function PutString($fileString, $string, $position, $length) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($string))       && (!is_string($string)))   trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($position))) trigger_error(VA_Error(1));
    if ((!is_int($length))       && (!is_string($length)))   trigger_error(VA_Error(1));
    
    $position--;

    if (strlen($string) < $length) {
        $string = str_pad($string, $length, chr(32));
    } else {
        $string = substr($string, 0, $length);
    }
    $fileString = substr_replace($fileString, $string, $position, $length);
    
    return $fileString;                     // Return the new string.
}


/**
* Va-Functions.inc.php :: PutChar()
*
* This function will put the passed integer into the file string at the given
* position. This is used for writing characters, or one byte integers.
*
*/
function PutChar($fileString, $int, $position) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($int))          && (!is_string($position))) trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($length)))   trigger_error(VA_Error(1));

    $position--;

    $int = MakeChar($int);
    $fileString = substr_replace($fileString, $int, $position, 1);

    return $fileString;                     // Return the new string.
}


/**
* Va-Functions.inc.php :: PutInteger()
*
* This function will put the passed integer into the file string at the given
* position. This is used for writing two byte integers. The default maximum
* value can be 32767, but if you set $long to 1 then the maximum is raised to
* 65535.
*
*/
function PutInteger($fileString, $int, $position, $long = 0) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($int))          && (!is_string($int)))      trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($position))) trigger_error(VA_Error(1));

    $position--;

    $int = MakeInteger($int, $long);
    $fileString = substr_replace($fileString, $int, $position, 2);

    return $fileString;                     // Return the new string.
}


/**
* Va-Functions.inc.php :: PutLongInt()
*
* This function will put the passed integer into the file string at the given
* position. This is used for writing four byte integers. The maximum value
* can be 2147483647, but if you set $long to 1, then the maximum is raise to
* 4294967295.
*
*/
function PutLongInt($fileString, $int, $position, $long = 0) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($int))          && (!is_string($int)))      trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($position))) trigger_error(VA_Error(1));

    $position--;

    $int = MakeLongInt($int, $long);
    $fileString = substr_replace($fileString, $int, $position, 4);

    return $fileString;                     // Return the new string.
}


/**
* Va-Functions.inc.php :: PutSingle()
*
* This function will put the passed single precision value into the file string
* at the given position.
*
*/
function PutSingle($fileString, $int, $position) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ((!is_int($int))          && (!is_string($int)))      trigger_error(VA_Error(1));
    if ((!is_int($position))     && (!is_string($position))) trigger_error(VA_Error(1));

// ** It has not been determined how to extract this value, let alone write it! **
}


/**
* Va-Functions.inc.php :: MakeChar()
*
* This function will take the value given and convert it into a one byte
* integer represented by ascii characters.
*
*/
function MakeChar($value) {
    if ((!is_int($value)) && (!is_string($value))) trigger_error(VA_Error(1));
    
    if ($value == 0)  return chr(0);
    if ($value > 256) trigger_error(VA_Error(2));
    
    $a = chr($value);                       // Set the variable to the ascii character.
    
    return $a;                              // Return the character.
}


/**
* Va-Functions.inc.php :: MakeInteger()
*
* This function will take the value given and convert it into a two byte
* integer represented by ascii characters. The maximum value can be 32767. If
* you set $long to 1, then you can create an integer with a value up to 65535.
*
*/
function MakeInteger($value, $long = 0) {
    if ((!is_int($value)) && (!is_string($value))) trigger_error(VA_Error(1));

    if ($value == 0)    return chr(0) . chr(0);
    if ($long == 0) {
        if ($value > 32767) trigger_error(VA_Error(2));         // This is the variable type maximum.
    } else {
        if ($value > 65535) trigger_error(VA_Error(2));         // This is the maximum that can be stored.
    }
    
    $b = intval($value / 256);              // Divide the value.
    $value = $value - ($b * 256);           // Subtract the variable from the value.
    $b = chr($b);                           // Set the variable to the ascii character.

    $a = chr($value);                       // Set the variable to the ascii character.

    return $a . $b;                         // Return the ascii representation.
}


/**
* Va-Functions.inc.php :: MakeLongInt()
*
* This function will take the value given and convert it into a four byte
* long integer represented by ASCII characters. The maximum value can be
* 2147483647. If you set $long to 1, then you can create a long integer with a
* value up to 4294967295.
*
*/
function MakeLongInt($value, $long = 0) {
    if ((!is_int($value)) && (!is_string($value))) trigger_error(VA_Error(1));

    if ($value == 0)         return chr(0) . chr(0) . chr(0) . chr(0);
    if ($long == 0) {
        if ($value > 2147483647) trigger_error(VA_Error(2));    // This is the variable type maximum.
    } else {
        if ($value > 4294967295) trigger_error(VA_Error(2));    // This is the maximum that can be stored.
    }
    
    $d = intval($value / 16777216);         // Divide the value.
    $value = $value - ($d * 16777216);      // Subtract the variable from the value.
    $d = chr($d);                           // Set the variable to the ascii character.

    $c = intval($value / 65536);            // Divide the value.
    $value = $value - ($c * 65536);         // Subtract the variable from the value.
    $c = chr($c);                           // Set the variable to the ascii character.
    
    $b = intval($value / 256);              // Divide the value.
    $value = $value - ($b * 256);           // Subtract the variable from the value.
    $b = chr($b);                           // Set the variable to the ascii character.
    
    $a = chr($value);                       // Set the variable to the ascii character.

    return $a . $b . $c . $d;               // Return the ascii representation.
}


/**
* Va-Functions.inc.php :: ReadConfig()
*
* This function will read a configuration file and place its variables into
* an array.
*
* The type of configuration file has comments designated by a semicolon, and
* variables are assigned like 'variable='value''.
*
*/
function ReadConfig($configFile) {
    if (!is_string($configFile)) trigger_error(VA_Error(1));
    
    // If the configfile is missing, return the default values.
    if (!is_file($configFile)) trigger_error('File is missing.');

    // Read the configfile into an array.
    $fp = fopen($configFile, 'r');          // Open the configfile.
    $x  = 1;

    while (!feof($fp)) {
        $line    = trim(fgets($fp));        // Read a line.
        $comment = trim(substr($line, 0, 1));
        if (($comment != ';') && ($line != '')) {
            $var[$x] = $line;           // Add the line into the array.
            $x++;                       // Increase the array. Used below too as a counter.
        }
    }

    fclose($fp);                            // Close the configfile.

    if (!isset($var)) return $config;  // Check to see if any values were read.

    // The configuration is now in an array, so lets set the proper variables
    // with the information.

    for ($y = 1; $y != $x; $y++) {                              // Examine the array and set new values.
        $pos   = strpos($var[$y], '=');                         // Determine the position of the equal symbol.
        $desc  = trim(strtolower(substr($var[$y], 0, $pos)));   // Determine the variable description.
        $value = trim(substr($var[$y], $pos + 1));              // Determine the variable value.
        $value = str_replace(' ', '', $value);                  // Remove any spaces.
        $config[$desc] = strtolower($value);
    }

    return $config;                         // Return the configuration array.
}


/**
* Va-Functions.inc.php :: ReadAt2kConfig($configFile)
*
* This function will read an AT2k script configuration file, which usually is
* stored in the BBS's TXT directory.
*
* The configuration file has comments designated by a semicolon, and values are
* positioned based (they must appear in the file in a certain order and are not
* assigned to variables).
*
*/
function ReadAt2kConfig($configFile) {
    if (!is_string($configFile)) trigger_error(VA_Error(1));
    
    // If the configfile is missing, send an error.
    if (!is_file($configFile)) trigger_error('File is missing.');

    // Read the configfile into an array.
    $fp = fopen($configFile, 'r');          // Open the configfile.
    $x  = 1;

    while (!feof($fp)) {
        $line    = trim(fgets($fp));        // Read a line.
        $comment = trim(substr($line, 0, 1));
        if (($comment != ';') && ($line != '')) {
            $var[$x] = $line;           // Add the line into the array.
            $x++;                       // Increase the array. Used below too as a counter.
        }
    }

    fclose($fp);                            // Close the configfile.
    
    return $var;
}


/**
* Va-Functions.inc.php :: FixHeartColors()
*
* This routine will take $string and replace any heart codes within the string
* with html color codes.
*
*/
function FixHeartColors($string) {
    if (!is_string($string)) trigger_error(VA_Error(1));
    
    // Check to see if the variable was passed, if not then return the string.
    if ($string == '') return $string;
    
  	$string = str_replace(chr(3) . '0', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . '1', "<font color=\"#52FFFF\">", $string);
   	$string = str_replace(chr(3) . '2', "<font color=\"#FFFF52\">", $string);
   	$string = str_replace(chr(3) . '3', "<font color=\"#AD00AD\">", $string);
   	$string = str_replace(chr(3) . '4', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . '5', "<font color=\"#52FF52\">", $string);
   	$string = str_replace(chr(3) . '6', "<font color=\"#FF5552\">", $string);
   	$string = str_replace(chr(3) . '7', "<font color=\"#5255FF\">", $string);
   	$string = str_replace(chr(3) . '8', "<font color=\"#AD5500\">", $string);
   	$string = str_replace(chr(3) . '9', "<font color=\"#00AAAD\">", $string);
   	$string = str_replace(chr(3) . 'a', "<font color=\"#00AA00\">", $string);
   	$string = str_replace(chr(3) . 'b', "<font color=\"#FF55FF\">", $string);
   	$string = str_replace(chr(3) . 'c', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'd', "<font color=\"#FFFF52\">", $string);
   	$string = str_replace(chr(3) . 'e', "<font color=\"#52FFFF\">", $string);
   	$string = str_replace(chr(3) . 'f', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'g', "<font color=\"#5255FF\">", $string);
   	$string = str_replace(chr(3) . 'h', "<font color=\"#FF5552\">", $string);
   	$string = str_replace(chr(3) . 'i', "<font color=\"#5255FF\">", $string);
   	$string = str_replace(chr(3) . 'j', "<font color=\"#FFFF52\">", $string);
   	$string = str_replace(chr(3) . 'k', "<font color=\"#52FFFF\">", $string);
   	$string = str_replace(chr(3) . 'l', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'm', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'n', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'o', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'p', "<font color=\"#AD0000\">", $string);
   	$string = str_replace(chr(3) . 'q', "<font color=\"#0000AD\">", $string);
   	$string = str_replace(chr(3) . 'r', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 's', "<font color=\"#FF5552\">", $string);
   	$string = str_replace(chr(3) . 't', "<font color=\"#5255FF\">", $string);
   	$string = str_replace(chr(3) . 'u', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'v', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'w', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'x', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'y', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'z', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'A', "<font color=\"#00AA00\">", $string);
   	$string = str_replace(chr(3) . 'B', "<font color=\"#FF55FF\">", $string);
   	$string = str_replace(chr(3) . 'C', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'D', "<font color=\"#FFFF52\">", $string);
   	$string = str_replace(chr(3) . 'E', "<font color=\"#52FFFF\">", $string);
   	$string = str_replace(chr(3) . 'F', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'G', "<font color=\"#5255FF\">", $string);
   	$string = str_replace(chr(3) . 'H', "<font color=\"#FF5552\">", $string);
   	$string = str_replace(chr(3) . 'I', "<font color=\"#5255FF\">", $string);
   	$string = str_replace(chr(3) . 'J', "<font color=\"#FFFF52\">", $string);
   	$string = str_replace(chr(3) . 'K', "<font color=\"#52FFFF\">", $string);
   	$string = str_replace(chr(3) . 'L', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'M', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'N', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'O', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'P', "<font color=\"#AD0000\">", $string);
   	$string = str_replace(chr(3) . 'Q', "<font color=\"#0000AD\">", $string);
   	$string = str_replace(chr(3) . 'R', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'S', "<font color=\"#FF5552\">", $string);
   	$string = str_replace(chr(3) . 'T', "<font color=\"#5255FF\">", $string);
   	$string = str_replace(chr(3) . 'U', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'V', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'W', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'X', "<font color=\"#ADAAAD\">", $string);
   	$string = str_replace(chr(3) . 'Y', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . 'Z', "<font color=\"#FFFFFF\">", $string);
   	$string = str_replace(chr(3) . '[', "<font color=\"#525552\">", $string);
   	$string = $string . '</font>';

   	return $string;                         // Return the new string.
}


/**
* Va-Functions.inc.php :: StripBBSCodes()
*
* This routine will take $string and remove any heart codes within the string.
* It also removes diamond codes.
*
*/
function StripBBSCodes($string) {
    if (!is_string($string)) trigger_error(VA_Error(1));
    
    // Check to see if the variable was passed, if not then return the string.
    if ($string == '') return $string;

    // Search for SBBS codes.
    for ($position = 0; is_int($position);) {
        $position = strpos($string, chr(1), 0);
        if (is_int($position)) $string = substr_replace($string, '', $position, 2);
    }

    // Search for hearts.
    for ($position = 0; is_int($position);) {
        $position = strpos($string, chr(3), 0);
        if (is_int($position)) $string = substr_replace($string, '', $position, 2);
    }

    // Search the file for diamonds.
    for ($position = 0; is_int($position);) {
        $position = strpos($string, chr(4), 0);
        if (is_int($position)) $string = substr_replace($string, '', $position, 2);
    }

   	return $string;                         // Return the new string.
}


/**
* Va-Functions.inc.php :: StripANSICodes()
*
* This function will strip all standard ANSI codes from the passed string.
*
*/
function StripANSICodes($string) {
    if (!is_string($string)) trigger_error(VA_Error(1));

    // Check to see if the variable was passed, if not then return the string.
    if ($string == '') return $string;

    $pattern = array(
                    "/\e\[(\d){0,3}[a|A|b|B|c|C|d|D]/",
                    "/\e\[((\?*?(\d){1,2})|((\d){1,2};{0,1})){0,2}[f|F|r|R]/",
                    "/\e\[((\?*?(\d){1,2})|((\d){1,2};{0,1})){0,2}[h|H]/",
                    "/\e\[(\d){0,1}[j|J]/",
                    "/\e\[(((\d){1,2};{0,1}){0,4})[m|M]/",
                    "/\e\[6[n|N]/",
                    "/\e\[((\d){1,2};{0,1}){0,3}[p|P]/",
                    "/\e\[[k|K|s|S|u|U]/"
                    );

    return preg_replace($pattern, '', $string); // Return the new string.
}


/**
* Va-Functions.inc.php :: StripRouting()
*
* This function will take $string and remove the lines that begin with a
* diamond code.
*
*/
function StripRouting($string) {
    if (!is_string($string)) trigger_error(VA_Error(1));
    
    // Check to see if the variable was passed, if not then return the string.
    if ($string == '') return $string;

    // Search for diamonds.
    for ($position = 0; is_int($position);) {
        $position = strpos($string, chr(4), 0);
        if (is_int($position)) {
            $endPosition = strpos($string, "\n", $position);
            $string      = substr_replace($string, '', $position, ($endPosition - $position));
        }
    }
    
    return $string;                         // Return the new string.
}


/**
* Va-Functions :: ConvertLongFilename()
*
* This function will convert a long filename into a DOS 8.3 filename.
*
*/
function ConvertLongFilename($filename, $dir = '') {
    if (!is_string($filename)) trigger_error(VA_Error(1));
    
    $pos  = strrpos($filename, '.');        // Find the period.
    if ($pos == 0) $pos = strlen($filename);
    $file = substr($filename, 0, $pos);     // Get the filename.
    if ($pos < strlen($filename)) {
        $ext = substr($filename, $pos + 1); // Get the extension.
        if (strlen($ext) > 3) $ext = substr($ext, 0, 3);
    } else {
        $ext = '';
    }
    if (strlen($file) < 8) return str_replace(' ', '_', $file . '.') . $ext;
    
    $pos = strpos($filename, '.');
    if ($pos <= 6) {
        $file = substr($file, 0, $pos);
    } else {
        $file = substr($file, 0, 6);
    }

    if ($dir != '') {
        $filename = $file . '~1.' . $ext;
        $filepath = str_replace('/', "\\", $dir . '/' . $filename);
        if (is_file($filepath)) {           // If that file already exists...
            for ($x = 1; (is_file($filepath) && ($x < 99)); $x++) {
                if ($x == 10) $file = substr($file, 0, 5);
                $filename = $file . '~' . $x . '.' . $ext;
                $filepath = str_replace('/', "\\", $dir . '/' . $filename);
            }
        }
    }
    
    return $filename;
}


/**
* Va-Functions.inc.php :: HtmlEncode()
*
* This function will take the passed line and add html href information for any
* links found. It will search for http://, ftp://, mailto: and telnet://.
*
*/
function HTMLEncode($string) {
    if (!is_string($string)) trigger_error(VA_Error(1));
    
    $string = ' ' . $string;
    $string = preg_replace(array(
                               "/( )(([\w\.\-]+))(@)([\w\.\-]+)\b/i",
                               "/((ftp(7?):\/\/)|(ftp\.))([\w\.\/\&\=\?\-~:]+)/",
                               "/(http(s?):\/\/)([\w\.\/\&\=\?\-~:#]+)/",
                               "/( )(www\.)([\w\.\/\&\=\?\-~:#]+)/",
                               "/(telnet:\/\/)([\w\.\/\&\=\?\-:]+)/"
                              ),
                         array(
                               " <a href=\"mailto:$3$4$5\"\">$3$4$5</a>",
                               "<a href=\"ftp$3://$4$5\" target=\"_blank\">ftp://$4$5</a>",
                               "<a href=\"http:$2//$3\" target=\"_blank\">$1$3</a>",
                               " <a href=\"http://$2$3\" target=\"_blank\">$2$3</a>",
                               "<a href=\"telnet://$2$3\">telnet://$2$3</a>"
                              ),
                         $string);

    return substr($string, 1);              // Return the encoded string, remove that inital space.
}


/**
* Va-Functions.inc.php :: ProgramTime()
*
* This function simply returns the current timestamp in microseconds.
*
*/
function ProgramTime() {
    $time = microtime();
    $time = explode (' ', $time);
    return $time[0] + $time[1];
}


/**
* Va-Functions.inc.php :: ConvertDateTime()
*
* This function will convert the many formats of dates and times into a
* standard format that VADV-PHP will use.
*
*/
function ConvertDateTime($string) {
    $date['date'] = 'Date';
    $date['time'] = 'Time';
    if ($string == '') return $date;
    
    $string = trim($string);
    $timeStamp = strtotime($string);

    if ($timeStamp == -1) {
        $pos = strpos($string, '19');
        $year = intval(substr($string, $pos, 4));
        while (($year < 1900) && ($pos !== FALSE)) {
            $pos = strpos($string, '19', $pos + 1);
            $year = intval(substr($string, $pos, 4));
        }
        if (($year < 1970) && ($year >= 1900)) {
            $newstring = substr_replace($string, $year + 100, $pos, 4);
            $timeStamp = strtotime($newstring);
        } else {
            $a = substr($string, 0, 10);
            $b = substr($string, 10, 9);
            $c = substr($string, 19, 5);
            $newstring = $a . $c . $b;
            $timeStamp = strtotime($newstring);
        }
        if ($timeStamp == -1) return $date;
    }

    $date['date'] = trim(date("M j, Y", $timeStamp));
    $date['time'] = trim(date("g:i A", $timeStamp));
    
    return $date;
}


/**
* Va-Functions.inc.php :: GetAccFlags()
*
* This function will take the long integer value of the access flags and
* convert it into a string format. The returned letters are all uppercase.
*
*/
function GetAccFlags($accFlagsInt) {
    if ((!is_int($accFlagsInt)) && (!is_string($accFlagsInt))) trigger_error(VA_Error(1));
    
    // Check to see if the variable was passed, if not then return nothing.
    if ($accFlagsInt == '') return '';
    
    $flags  = '';                           // Flags start off as none.
    $num    = 33554432;                     // This is letter Z's value as a user flag.

    for ($letter = 90; $num >= 1; $letter--) {  // It uses capital letters (letter 90).
        $tmp = intval($accFlagsInt / $num);
        if ($tmp >= 1) {
            $flags       = $flags . chr($letter);
            $accFlagsInt = $accFlagsInt - $num;
        }
        $num = $num / 2;
    }

    return strrev($flags);                  // Return the access flags.
}


/**
* Va-Functions.inc.php :: MakeAccFlags()
*
* This function will take the string representation of user access flags and
* convert it into an integer suitable for writing to userfile.dat
*
*/
function MakeAccFlags($accFlags) {
    if ((!is_int($accFlags)) && (!is_string($accFlags))) trigger_error(VA_Error(1));

    $accFlags    = strtoupper(str_replace(' ', '', $accFlags)); // Remove any white space and convert it to upper case.
    $accFlagsInt = 0;
    
    for ($x = strlen($accFlags); $x >= 0; $x--) {
        $letter = substr($accFlags, $x, 1); // Get a character.
        if (((ord($letter) >= 65) && (ord($letter) <= 90)) &&   // It must be a letter, or ignore it.
           (strpos($accFlags, $letter) === $x))                 // If the letter does not occur before this one...
                $accFlagsInt = $accFlagsInt + pow(2, ord($letter) - 65);
    }
    
    return $accFlagsInt;                    // Return the integer.
}


/**
* Va-Functions.inc.php :: MakeFlags()
*
* This function will take the string $accFlags and convert it to the 26 character
* representation which is writable to userfile.dat.
*
*/
function MakeFlags($accFlags) {
    if ((!is_int($accFlags)) && (!is_string($accFlags))) trigger_error(VA_Error(1));

    $accFlags = strtoupper(str_replace(' ', '', $accFlags));    // Remove any white space and convert it to upper case.
    $newFlags = str_pad('', 26, ' ');                           // Create the blank string.

    // Check to see if the variable was passed, if not then return nothing.
    if ($accFlags == '') return $newFlags;
    
    for ($x = strlen($accFlags); $x >= 0; $x--) {
        $letter = substr($accFlags, $x, 1); // Get a character.
        if ((ord($letter) >= 65) && (ord($letter) <= 90))       // It must be a letter, or ignore it.
            $newFlags = substr_replace($newFlags, $letter, ord($letter) - 65, 1);
    }
    
    return $newFlags;                       // Return the new string.
}


/**
* Va-Functions.inc.php :: UpdateWhosOnline()
*
* This function will maintain the who's online listing for VADV-PHP. It is
* automatically called by va-functions.inc.php.
*
*/
function UpdateWhosOnline() {
    global $VA_SYSPATHS;                    // Include global variable.
    global $USER;

    $file = $VA_SYSPATHS['data'] . '/WEBCALLER.LST';    // Database filename.
    $fileString = ReadCfgFile($file);       // Read the file.

    // Read through the file looking for the current user's IP, and also
    // expired users. Users expire after 5 mins (300s).
    $user   = array();
    $online = array();
    $offset = 0;
    for ($x = 1; $offset < strlen($fileString); $x++) {
        $user['session'] = trim(GetString($fileString, 1 + $offset, 32));
        $user['ip']      = trim(GetString($fileString, 33 + $offset, 15));
        $user['time']    = intval(trim(GetString($fileString, 48 + $offset, 12)));
        $user['userno']  = intval(trim(GetInteger($fileString, 60)));
        $user['handle']  = trim(GetString($fileString, 62 + $offset, 40));
        if (($user['userno'] == $USER['userno']) || (session_id() == $user['session']) || (time() - $user['time'] > 300)) {
            $fileString = substr_replace($fileString, '', $offset, 101);
            $x--;
        } else {
            $offset = 101 * $x;
            $online[$x]['ip']      = $user['ip'];
            $online[$x]['time']    = $user['time'];
            $online[$x]['userno']  = $user['userno'];
            $online[$x]['handle']  = $user['handle'];
        }
    }

    // Add the current user's entry to the end.
    $ip     = str_pad($_SERVER['REMOTE_ADDR'], 15);
    $time   = str_pad(time() + 300, 12);
    $userno = MakeInteger($USER['userno']);
    $handle = str_pad($USER['handle'], 40);

    $fileString = $fileString . session_id() . $ip . $time . $userno . $handle;
    WriteCfgFile($file, $fileString);

    $online[$x]['ip']     = trim($ip);
    $online[$x]['time']   = trim($time);
    $online[$x]['userno'] = trim($USER['userno']);
    $online[$x]['handle'] = trim($handle);
    
    // Organize the array alphabetically by handle.
    foreach ($online as $row) {
        $sortarr[] = $row['handle'];
    }
    array_multisort($sortarr, $online);
    array_unshift($online, $x);

    return $online;                         // Return the online users.
}


/**
* Va-Functions.inc.php :: WriteLog()
*
* This function will log a line into the specified file which is stored in
* the BBS's data directory. The logfile is a filename. MaxSize is an optional
* parameter which can be used to reset the log file if it is larger than the
* value (in bytes). 
*
*/
function WriteLog($logFile, $line, $maxSize = 0) {
    global $VA_SYSPATHS;

    $logFile = $VA_SYSPATHS['data'] . '/' . $logFile;

    // If log is larger than $maxSize then reset it.
    if (($maxSize != 0) && (is_file($logFile)) && (filesize($logFile) >= $maxSize)) unlink($logFile);
    
    if (!is_file($logFile)) {
        $fp = fopen($logFile, 'w');
        flock($fp, 2);
        fwrite($fp, "\r\n");
        fwrite($fp, "VADV-PHP Event Log\r\n");
        fwrite($fp, "-------------------------------------------------------------------------------\r\n");
        fwrite($fp, "Date           Time     IP Address        Description\r\n");
        fwrite($fp, "-------------------------------------------------------------------------------\r\n");
        flock($fp, 3);
        fclose($fp);
    }

    $fp = fopen($logFile, 'a');
    flock($fp, 2);                          // Lock the file.

    $ip = '[' . $_SERVER['REMOTE_ADDR'] . ']';
    $ip = str_pad($ip, 16, ' ');

    $date   = date('D') . ' ' . date('Y') . '-' . date('m') . '-' . date('d');
    $time   = date('H') . ':' . date('i') . ':' . date('s');
    $line   = $date . ' ' . $time . ' ' . $ip . '  ' . $line . "\r\n";
    $status = fwrite($fp, $line);           // Write the line to the file.

    flock($fp, 3);                          // Unlock the file.
    fclose($fp);                            // Close the file.
    
    return $status;                         // Return the result.
 }


/**
* Va-Functions.inc.php :: SendOneLiner()
*
* This function will send a one-liner to the specified node, or to all active
* nodes. If the userNo is 0, then it will send the message to any user. If
* the node is set to all, it will send it to all active nodes.
*
*/
function SendOneLiner($message, $userNo = 0, $node = 'All') {
    if (!is_string($message)) trigger_error(VA_Error(1));
    if ((!is_int($userNo)) && (!is_string($userNo))) trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                                // Include global variable.
    global $includeDir;
    include_once($includeDir . 'va-control.inc.php');   // Include control.dat functions.
    include_once($includeDir . 'va-main.inc.php');      // Include main configuration.
    global $VA_MAIN;
    
    // Truncate the message if too long.
    if (strlen($message) > 75) $message = substr($message, 0, 75);
    // Add coloring.
    $message = chr(3) . '6*** ' . chr(3) . '1' . $message;

    $status = FALSE;
    if (($userNo == 0) && ($node != 'All')) {
        $file = $VA_SYSPATHS['control'] . '/in' . $node . '.que';
        $fp = fopen($file, 'a');            // Open the file.
        flock($fp, 2);                      // Lock the file.
        $status = fwrite($fp, $message);    // Write the message.
        flock($fp, 3);                      // Unlock the file.
        fclose($fp);
    } else {
        if ($userNo > 0) {
            for ($x = 0; $x < $VA_MAIN['maxchannelno']; $x++) {
                $channel = GetControl($x);
                if ($channel['userno'] == $userNo) {
                    $file = $VA_SYSPATHS['control'] . '/in' . $x . '.que';
                    $fp = fopen($file, 'a');            // Open the file.
                    flock($fp, 2);                      // Lock the file.
                    $status = fwrite($fp, $message);    // Write the message.
                    flock($fp, 3);                      // Unlock the file.
                    fclose($fp);
                }
            }
        } else {
            for ($x = 0; $x < $VA_MAIN['maxchannelno']; $x++) {
                $channel = GetControl($x);
                if ($channel['userno'] > 0) {
                    $file = $VA_SYSPATHS['control'] . '/in' . $x . '.que';
                    $fp = fopen($file, 'a');            // Open the file.
                    flock($fp, 2);                      // Lock the file.
                    $status = fwrite($fp, $message);    // Write the message.
                    flock($fp, 3);                      // Unlock the file.
                    fclose($fp);
                }
            }
        }
    }

    return $status;                         // Return the status.
}


/**
* Va-Functions.inc.php :: GetCurrentURL()
*
* This function simply returns the full URL of the current script. If $query is
* set to 0, then it will return the current URL without the query string.
*
*/
function GetCurrentURL($query = 1) {
    $url = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF'];
    if ((isset($_SERVER['QUERY_STRING'])) &&
        ($query != 0) &&
        (trim($_SERVER['QUERY_STRING']) != '')) $url = $url . '?' . $_SERVER['QUERY_STRING'];

    return $url;                            // Return the url.
}


/**
* Va-Funtions.inc.php :: RedirectUser()
*
* This function will redirect the user to the specified URL. It uses a primary
* base of JavaScript with a backup meta tag for those browsers that do not
* support JavaScript. Note that when this is called, the current page will
* exit and the redirection will take place. You may set delay the redirection
* a specific number of seconds by using the optional $delay variable.
*
*/
function RedirectUser($url, $delay = 0) {
    if (!is_string($url)) return FALSE;
?>
<html>

<head>
    <title></title>
    <noscript>
        <meta http-equiv="refresh" content="<?php print $delay + 2; ?>; URL=<?php print $url; ?>" />
    </noscript>

    <script language="JavaScript">
    <!--
    var sTargetURL = "<?php print $url; ?>";

    function doRedirect() {
        setTimeout("window.location.href = sTargetURL", <?php print $delay + 2; ?>*1000);
    }
    //-->
    </script>

    <script language="JavaScript1.1">
    <!--
    function doRedirect() {
        setTimeout("window.location.replace(sTargetURL)", <?php print $delay; ?>*1000);
    }
    doRedirect();
    //-->
    </script>
</head>

<body onload="doRedirect()"></body>

</html>
<?php
    die();
}


/**
* Va-Functions.inc.php :: array_stripslashes()
*
* This function will remove slashes from an array.
*
*/
function Array_StripSlashes(&$array) {
    if (!is_array($array)) return;
    while (list($key) = each($array)) {
        if (is_array($array[$key])) {
            stripslashes_array($array[$key]);
        } else {
            $array[$key] = stripslashes($array[$key]);
        }
    }
}


/**
* Va-Functions.inc.php :: SendFile()
*
* This function will send the specified file to the user using the fpassthru()
* function.
*
*/
function SendFile($file) {
    if (!is_file($file)) return FALSE;

    $status   = FALSE;                      // Set the status to default to false.
    $filename = basename($file);            // Get the filename.
    $filesize = filesize($file);            // Get the filesize.
    $filesizemb = intval($filesize / 1048576);

    // Determine MIME type.
    $pos = strrpos($filename, '.');
    $ext = strtolower(substr($filename, $pos));
    switch ($ext) {
        case '.ai':   $mimetype='application/postscript';   $encode='7bit';     break;
        case '.avi':  $mimetype='video/x-msvideo';          $encode='base64';   break;
        case '.doc':  $mimetype='application/msword';       $encode='base64';   break;
        case '.eps':  $mimetype='application/postscript';   $encode='7bit';     break;
        case '.exe':  $mimetype='application/octet-stream'; $encode='base64';   break;
        case '.gif':  $mimetype='image/gif';                $encode='base64';   break;
        case '.htm':  $mimetype='text/html';                $encode='7bit';     break;
        case '.html': $mimetype='text/html';                $encode='7bit';     break;
        case '.jpe':  $mimetype='image/jpeg';               $encode='base64';   break;
        case '.jpeg': $mimetype='image/jpeg';               $encode='base64';   break;
        case '.jpg':  $mimetype='image/jpeg';               $encode='base64';   break;
        case '.mpe':  $mimetype='video/mpeg';               $encode='base64';   break;
        case '.mpeg': $mimetype='video/mpeg';               $encode='base64';   break;
        case '.mov':  $mimetype='video/quicktime';          $encode='base64';   break;
        case '.mpg':  $mimetype='video/mpeg';               $encode='base64';   break;
        case '.pdf':  $mimetype='application/pdf';          $encode='base64';   break;
        case '.ps':   $mimetype='application/postscript';   $encode='7bit';     break;
        case '.qt':   $mimetype='video/quicktime';          $encode='base64';   break;
        case '.rtf':  $mimetype='application/rtf';          $encode='7bit';     break;
        case '.tar':  $mimetype='application/octet-stream'; $encode='base64';   break;
        case '.tif':  $mimetype='image/tiff';               $encode='base64';   break;
        case '.tiff': $mimetype='image/tiff';               $encode='base64';   break;
        case '.txt':  $mimetype='text/plain';               $encode='7bit';     break;
        case '.wav':  $mimetype='audio/x-wav';              $encode='base64';   break;
        case '.zip':  $mimetype='application/zip';          $encode='base64';   break;
        default:      $mimetype='application/octet-stream'; $encode='base64';   break;
    }

    // Set some new time limits and file sizes. (Do not think this is necessary)
    ini_set('max_execution_time', 240);
    ini_set('memory_limit', 4 + $filesizemb . 'M');

    header('Cache-Control: ');
    header('Pragma: ');
    header('Content-type: ' . $mimetype);
    header('Content-Disposition: attachment; filename="' . $filename . '"');
    header('Content-transfer-encoding: ' . $encode);
    header('Content-length: ' . $filesize);

    $fp = fopen($file, 'rb');               // Open the file to send.
    $status = fpassthru($fp);               // Send the file.

    return $status;                         // Return the status.
}


/**
* Va-Functions.inc.php :: VA_Error()
*
* This function will return the specified error message.
*
*/
function VA_Error($errNum) {
    if ((!is_int($errNum)) && (!is_string($errNum))) $errNum = 4;

    switch ($errNum) {
        case 0:
            $errMsg = 'File not found.';
            break;
        case 1:
            $errMsg = 'Variable is not proper type.';
            break;
        case 2:
            $errMsg = 'Value is out of range.';
            break;
        case 3:
            $errMsg = 'Invalid Configuration.';
            break;
        case 4:
            $errMsg = 'Unknown Error.';
            break;
    }
    
    return $errMsg;                         // Return the error message.
}


/**
* Va-Functions.inc.php :: VA_LoginError()
*
* This function will display a message stating that the user needs to login to
* access a certain area and give a link to the generic login page.
*
*/
function VA_LoginError() {
    global $CONFIG;                         // Include global variable.
    global $includeDir;
?>
<html>

<head>
<title>Login Authentication Error</title>
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8559-1" />
<meta name="copyright" content="Copyright  2003, Steve Winn" />
<link rel="stylesheet" type="text/css" href="./../VADVPHP.css" />
</head>

<body bgcolor="#000000">

<div align="center">
    <center>
    <table border="1" width="640" cellspacing="0" cellpadding="5" bgcolor="#000080" bordercolor="#FFFFFF">
        <tr>
            <td valign="top">
                <p class="yellowTitle" align="center">Login Authentication Error</p>
                <table cellspacing="1" width="100%">
                    <tr>
                        <td valign="top">
                            <p class="navBar" align="center">The area you are attempting to access
                            requires that you login first.</p>
                            <p align="center"><a class="return" href="<?php print $includeDir; ?>./../bbslogin.php?url=<?php print urlencode(GetCurrentURL()); ?>">Click here to login.</a></p>
                        </td>
                    </tr>
                </table>
                <p align="center"><span class="navBar">This page is powered
                by <a class="navBar" href="http://www.vadvphp.com">VADV-PHP</a></span></p>
                <p align="center" class="infoBarBody">If you experience any problems
                with this website or need help, contact the
                <a class="infoBarBody" href="mailto:<?php print $CONFIG['webmaster']; ?>">webmaster</a>.<br />
                VADV-PHP Copyright  2002-<?php print date('Y'); ?> Aspect
                Technologies. All Rights Reserved.</p>
            </td>
        </tr>
    </table>
    </center>
</div>

</body>

</html>
<?php
}


/**
* Va-Functions.inc.php :: CustomErrorHandler()
*
* This is a custom error handler in case all hell breaks loose.
*
*/
function CustomErrorHandler($errNumber, $errDescript, $errFile, $errLine) {
    $pos  = strrpos($errFile, chr(92)) + 1;
    $file = substr($errFile, $pos);
    print "<h2>An error has occurred! Please try again later.</h2>\n";
    print "<h4>Press your BACK button to return to the previous page.</h4>\n";
    print "<pre>Error number: $errNumber\n";
    print "Description: $errDescript\n";
    print "Script name: $file\n";
    print "Line number: $errLine</pre>\n";
    die();
}

set_error_handler('CustomErrorHandler');    // Execute the default error handler.
error_reporting(E_ALL);

?>
