<?php
/*
 * ----------------------------------------------------------------------------
 * Va-MultiFBX.inc.php                                               05/07/2003
 * This script will retrieve all of the informaton related to the configured
 * multi-feedback settings that are found in MULTIFBX.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2003 Steve Winn. All Rights Reserved.
 *
 * The source code contains a fair amount of comments, which may aid anyone
 * attempting to follow the program or to modify it.
 *
 * For licensing and copyright information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');                    // Get include directory.
require_once($includeDir . 'va-functions.inc.php'); // A VADV-PHP script.


/**
* Va-MultiFBX.inc.php :: GetMultiFBX()
*
* Retrieve all of the doors information and store it into a multidimensional
* array.
*
* Array format: $multifbx[1]['setting']
*               $multifbx[0] = Total number of multi-feeback settings.
*
*/
function GetMultiFBX() {
    $fileString = ReadMultiFBX();
    if (!$fileString) trigger_error(VA_Error(3));

    $multifbx = array();                    // Create the array.
    $offset   = 0;

    for ($x = 0; $offset < strlen($fileString); $x++) {
        $multifbx[$x + 1]['desc']       = GetString($fileString, 1 + $offset, 40);
        $multifbx[$x + 1]['handle']     = GetString($fileString, 41 + $offset, 40);
        $multifbx[$x + 1]['userno']     = GetLongInt($fileString, 81 + $offset);
        $multifbx[$x + 1]['netaddress'] = GetString($fileString, 85 + $offset, 60);
        $multifbx[$x + 1]['netno']      = GetInteger($fileString, 165 + $offset);
        $offset = 167 * $x;
    }

    $multifbx[0] = $x;                      // Put the number of doors in position 0.

    return $multifbx;                       // Return the multi-feedback settings.
}


/**
* Va-MultiFBX.inc.php :: PutMultiFBX()
*
* This function will change the multifbx.cfg to match the configuration variable
* passed. It then returns the new file string. The configuration variable MUST
* be in the format of $variable[1]['setting'].
*
*/
function PutMultiFBX($fileString, $configVariable) {
    if (!is_string($fileString))    trigger_error(VA_Error(1));
    if (!is_array($configVariable)) trigger_error(VA_Error(1));

    $length  = count($configVariable);      // Get the number of elements in the config variable.
    $pointer = 0;                           // Reset the pointer.

    for ($x = 0; $pointer < $length; $x++) {
        $total = intval(strlen($fileString) / 167);
        if ($x > $total - 1) $fileString = $fileString . str_pad('', 167, chr(0));

        $offset = 167 * $x;
        if (isset($configVariable[$x + 1])) {
            $pointer++;
            if (isset($configVariable[$x + 1]['desc']))       $fileString = PutString($fileString, $configVariable[$x + 1]['desc'], 1 + $offset, 40);
            if (isset($configVariable[$x + 1]['handle']))     $fileString = PutString($fileString, $configVariable[$x + 1]['handle'], 41 + $offset, 40);
            if (isset($configVariable[$x + 1]['userno']))     $fileString = PutLongInt($fileString, $configVariable[$x + 1]['userno'], 81 + $offset);
            if (isset($configVariable[$x + 1]['netaddress'])) $fileString = PutString($fileString, $configVariable[$x + 1]['netaddress'], 85 + $offset, 60);
            if (isset($configVariable[$x + 1]['netno']))      $fileString = PutInteger($fileString, $configVariable[$x + 1]['netno'], 165 + $offset);
        }
    }

    return $fileString;                     // Return the new file string.
}


/**
* Va-MultiFBX.inc.php :: ReadMultiFBX()
*
* This function will read the multifbx.cfg file.
*
*/
function ReadMultiFBX() {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/multifbx.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    return $fileString;                     // Return the file string.
}


/**
* Va-MultiFBX.inc.php :: WriteMultiFBX()
*
* This function will write a new multifbx.cfg file based on the file string
* passed to it. If the file string is not in the correct format, then your
* multifbx.cfg file will become corrupt and the BBS could stop functioning!
*
*/
function WriteMultiFBX($fileString) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/multifbx.cfg';
    $status = WriteCfgFile($file, $fileString);

    return $status;                         // Return the result status.
}


// Retrieve the muli-feedback settings automatically.
$VA_MULTIFBX = GetMultiFBX();

?>
