<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Stats.inc.php                                                  05/02/2003
 * This script contains functions used to read and write to the statistical
 * data files of VADV.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2003 Steve Winn. All Rights Reserved.
 *
 * The source code contains a fair amount of comments, which may aid anyone
 * attempting to follow the program or to modify it.
 *
 * For licensing and copyright information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');                    // Get include directory.
require_once($includeDir . 'va-functions.inc.php'); // A VADV-PHP script.


/**
* Va-Stats.inc.php :: GetStats()
*
* This function will retrieve the VADV statistics.
*
*/
function GetStats() {
    $fileString = ReadStats();
    if (!$fileString) trigger_error(VA_Error(3));
    
    $stats = array();
    $stats['callstotal']     = GetLongInt($fileString, 1);
    $stats['callstoday']     = GetLongInt($fileString, 5);
    $stats['netcallstotal']  = GetLongInt($fileString, 9);
    $stats['netcallstoday']  = GetLongInt($fileString, 13);
    $stats['activemintotal'] = GetLongInt($fileString, 17);
    $stats['activemintoday'] = GetLongInt($fileString, 21);
    $stats['idlemintotal']   = GetLongInt($fileString, 25);
    $stats['idlemintoday']   = GetLongInt($fileString, 29);
    $stats['daysonline']     = GetLongInt($fileString, 41);
    $stats['downloadstotal'] = GetLongInt($fileString, 45);
    $stats['downloadstoday'] = GetLongInt($fileString, 49);
    $stats['uploadstotal']   = GetLongInt($fileString, 53);
    $stats['uploadstoday']   = GetLongInt($fileString, 57);
    $stats['poststotal']     = GetLongInt($fileString, 61);
    $stats['poststoday']     = GetLongInt($fileString, 65);
    $stats['emailstotal']    = GetLongInt($fileString, 69);
    $stats['emailstoday']    = GetLongInt($fileString, 73);
    // The webcallstotal variable is not a standard VADV entry.. it is created
    // by VADV-PHP!
    $stats['webcallstotal']  = GetLongInt($fileString, 77);
    
    return $stats;                          // Return the statistics.
}


/**
* Va-Stats.inc.php :: PutStats()
*
* This function will write a new file string based on the configuration variable
* passed to it. The new file string can be used to create a new configuration
* file, but you must be careful that the configuration variable is valid.
*
*/
function PutStats($fileString, $configVariable) {
    if (!is_string($fileString))    trigger_error(VA_Error(1));
    if (!is_array($configVariable)) trigger_error(VA_Error(1));

    if (isset($configVariable['callstotal']))     $fileString = PutLongInt($fileString, $configVariable['callstotal'], 1);
    if (isset($configVariable['callstoday']))     $fileString = PutLongInt($fileString, $configVariable['callstoday'], 5);
    if (isset($configVariable['netcallstotal']))  $fileString = PutLongInt($fileString, $configVariable['netcallstotal'], 9);
    if (isset($configVariable['netcallstoday']))  $fileString = PutLongInt($fileString, $configVariable['netcallstoday'], 13);
    if (isset($configVariable['activemintotal'])) $fileString = PutLongInt($fileString, $configVariable['activemintotal'], 17);
    if (isset($configVariable['activemintoday'])) $fileString = PutLongInt($fileString, $configVariable['activemintoday'], 21);
    if (isset($configVariable['idlemintotal']))   $fileString = PutLongInt($fileString, $configVariable['idlemintotal'], 25);
    if (isset($configVariable['idlemintoday']))   $fileString = PutLongInt($fileString, $configVariable['idlemintoday'], 29);
    if (isset($configVariable['daysonline']))     $fileString = PutLongInt($fileString, $configVariable['daysonline'], 41);
    if (isset($configVariable['downloadstotal'])) $fileString = PutLongInt($fileString, $configVariable['downloadstotal'], 45);
    if (isset($configVariable['downloadstoday'])) $fileString = PutLongInt($fileString, $configVariable['downloadstoday'], 49);
    if (isset($configVariable['uploadstotal']))   $fileString = PutLongInt($fileString, $configVariable['uploadstotal'], 53);
    if (isset($configVariable['uploadstoday']))   $fileString = PutLongInt($fileString, $configVariable['uploadstoday'], 57);
    if (isset($configVariable['poststotal']))     $fileString = PutLongInt($fileString, $configVariable['poststotal'], 61);
    if (isset($configVariable['poststoday']))     $fileString = PutLongInt($fileString, $configVariable['poststoday'], 65);
    if (isset($configVariable['emailstotal']))    $fileString = PutLongInt($fileString, $configVariable['emailstotal'], 69);
    if (isset($configVariable['emailstoday']))    $fileString = PutLongInt($fileString, $configVariable['emailstoday'], 73);
    // The webcallstotal variable is not a standard VADV entry.. it is created
    // by VADV-PHP!
    if (isset($configVariable['webcallstotal']))  $fileString = PutLongInt($fileString, $configVariable['webcallstotal'], 77);
    
    return $fileString;                     // Return the new file string.
}


/**
* Va-Stats.inc.php :: ReadStats()
*
* This function will read the statsvbbs.dat file.
*
*/
function ReadStats() {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['data'] . '/statvbbs.dat';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.
    
    return $fileString;                     // Return the file string.
}


/**
* Va-Stats.inc.php :: WriteStats()
*
* This function will write a new statvbbs.dat file based on the file string
* passed to it. If the file string is not in the correct format, then your
* statvbbs.dat file will become corrupt and the BBS could stop functioning!
*
*/
function WriteStats($fileString) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['data'] . '/statvbbs.dat';
    $status = WriteCfgFile($file, $fileString);

    return $status;                         // Return the result status.
}


/**
* Va-Stats.inc.php :: GetHourStats()
*
* This function will retrieve the VADV hour statistics.
*
*/
function GetHourStats() {
    $fileString = ReadHourStats();

    $hourStats = array();
    $hourStats['12am'] = GetLongInt($fileString, 1);
    $hourStats['1am']  = GetLongInt($fileString, 5);
    $hourStats['2am']  = GetLongInt($fileString, 9);
    $hourStats['3am']  = GetLongInt($fileString, 13);
    $hourStats['4am']  = GetLongInt($fileString, 17);
    $hourStats['5am']  = GetLongInt($fileString, 21);
    $hourStats['6am']  = GetLongInt($fileString, 25);
    $hourStats['7am']  = GetLongInt($fileString, 29);
    $hourStats['8am']  = GetLongInt($fileString, 33);
    $hourStats['9am']  = GetLongInt($fileString, 37);
    $hourStats['10am'] = GetLongInt($fileString, 41);
    $hourStats['11am'] = GetLongInt($fileString, 45);
    $hourStats['12pm'] = GetLongInt($fileString, 49);
    $hourStats['1pm']  = GetLongInt($fileString, 53);
    $hourStats['2pm']  = GetLongInt($fileString, 57);
    $hourStats['3pm']  = GetLongInt($fileString, 61);
    $hourStats['4pm']  = GetLongInt($fileString, 65);
    $hourStats['5pm']  = GetLongInt($fileString, 69);
    $hourStats['6pm']  = GetLongInt($fileString, 73);
    $hourStats['7pm']  = GetLongInt($fileString, 77);
    $hourStats['8pm']  = GetLongInt($fileString, 81);
    $hourStats['9pm']  = GetLongInt($fileString, 85);
    $hourStats['10pm'] = GetLongInt($fileString, 89);
    $hourStats['11pm'] = GetLongInt($fileString, 93);

    return $hourStats;                      // Return the statistics.
}


/**
* Va-Stats.inc.php :: PutHourStats()
*
* This function will write a new file string based on the configuration variable
* passed to it. The new file string can be used to create a new configuration
* file, but you must be careful that the configuration variable is valid.
*
*/
function PutHourStats($fileString, $configVariable) {
    if (!is_string($fileString))    trigger_error(VA_Error(1));
    if (!is_array($configVariable)) trigger_error(VA_Error(1));

    if (isset($configVariable['12am'])) $fileString = PutLongInt($fileString, $configVariable['12am'], 1);
    if (isset($configVariable['1am']))  $fileString = PutLongInt($fileString, $configVariable['1am'], 5);
    if (isset($configVariable['2am']))  $fileString = PutLongInt($fileString, $configVariable['2am'], 9);
    if (isset($configVariable['3am']))  $fileString = PutLongInt($fileString, $configVariable['3am'], 13);
    if (isset($configVariable['4am']))  $fileString = PutLongInt($fileString, $configVariable['4am'], 17);
    if (isset($configVariable['5am']))  $fileString = PutLongInt($fileString, $configVariable['5am'], 21);
    if (isset($configVariable['6am']))  $fileString = PutLongInt($fileString, $configVariable['6am'], 25);
    if (isset($configVariable['7am']))  $fileString = PutLongInt($fileString, $configVariable['7am'], 29);
    if (isset($configVariable['8am']))  $fileString = PutLongInt($fileString, $configVariable['8am'], 33);
    if (isset($configVariable['9am']))  $fileString = PutLongInt($fileString, $configVariable['9am'], 37);
    if (isset($configVariable['10am'])) $fileString = PutLongInt($fileString, $configVariable['10am'], 41);
    if (isset($configVariable['11am'])) $fileString = PutLongInt($fileString, $configVariable['11am'], 45);
    if (isset($configVariable['12pm'])) $fileString = PutLongInt($fileString, $configVariable['12pm'], 49);
    if (isset($configVariable['1pm']))  $fileString = PutLongInt($fileString, $configVariable['1pm'], 53);
    if (isset($configVariable['2pm']))  $fileString = PutLongInt($fileString, $configVariable['2pm'], 57);
    if (isset($configVariable['3pm']))  $fileString = PutLongInt($fileString, $configVariable['3pm'], 61);
    if (isset($configVariable['4pm']))  $fileString = PutLongInt($fileString, $configVariable['4pm'], 65);
    if (isset($configVariable['5pm']))  $fileString = PutLongInt($fileString, $configVariable['5pm'], 69);
    if (isset($configVariable['6pm']))  $fileString = PutLongInt($fileString, $configVariable['6pm'], 73);
    if (isset($configVariable['7pm']))  $fileString = PutLongInt($fileString, $configVariable['7pm'], 77);
    if (isset($configVariable['8pm']))  $fileString = PutLongInt($fileString, $configVariable['8pm'], 81);
    if (isset($configVariable['9pm']))  $fileString = PutLongInt($fileString, $configVariable['9pm'], 85);
    if (isset($configVariable['10pm'])) $fileString = PutLongInt($fileString, $configVariable['10pm'], 89);
    if (isset($configVariable['11pm'])) $fileString = PutLongInt($fileString, $configVariable['11pm'], 93);

    return $fileString;                     // Return the new file string.

}


/**
* Va-Stats.inc.php :: ReadHourStats()
*
* This function will read the hour.dat file.
*
*/
function ReadHourStats() {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['data'] . '/hour.dat';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    return $fileString;                     // Return the file string.
}


/**
* Va-Stats.inc.php :: WriteHourStats()
*
* This function will write a new hours.dat file based on the file string passed
* to it. If the file string is not in the correct format, then your hours.dat
* file will become corrupt and the BBS could stop functioning!
*
*/
function WriteHourStats($fileString) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['data'] . '/hour.dat';
    $status = WriteCfgFile($file, $fileString);

    return $status;                         // Return the result status.
}


/**
* Va-Stats.inc.php :: GetConnectionStats()
*
* This function will retrieve the VADV connection statistics.
*
*/
function GetConnectionStats() {
    $fileString = ReadConnectionStats();

    $connectStats = array();
    $connectStats['1200']  = GetLongInt($fileString, 5);
    $connectStats['2400']  = GetLongInt($fileString, 9);
    $connectStats['9600']  = GetLongInt($fileString, 13);
    $connectStats['14400'] = GetLongInt($fileString, 17);
    $connectStats['19200'] = GetLongInt($fileString, 21);
    $connectStats['24000'] = GetLongInt($fileString, 25);
    $connectStats['28800'] = GetLongInt($fileString, 29);

    return $connectStats;                      // Return the statistics.
}


/**
* Va-Stats.inc.php :: PutConnectionStats()
*
* This function will write a new file string based on the configuration variable
* passed to it. The new file string can be used to create a new configuration
* file, but you must be careful that the configuration variable is valid.
*
*/
function PutConnectionStats($fileString, $configVariable) {
    if (!is_string($fileString))    trigger_error(VA_Error(1));
    if (!is_array($configVariable)) trigger_error(VA_Error(1));

    if (isset($configVariable['1200']))  $fileString = PutLongInt($fileString, $configVariable['1200'], 5);
    if (isset($configVariable['2400']))  $fileString = PutLongInt($fileString, $configVariable['2400'], 9);
    if (isset($configVariable['9600']))  $fileString = PutLongInt($fileString, $configVariable['9600'], 13);
    if (isset($configVariable['14400'])) $fileString = PutLongInt($fileString, $configVariable['14400'], 17);
    if (isset($configVariable['19200'])) $fileString = PutLongInt($fileString, $configVariable['19200'], 21);
    if (isset($configVariable['24000'])) $fileString = PutLongInt($fileString, $configVariable['24000'], 25);
    if (isset($configVariable['28800'])) $fileString = PutLongInt($fileString, $configVariable['28800'], 29);

    return $fileString;                     // Return the new file string.

}


/**
* Va-Stats.inc.php :: ReadConnectionStats()
*
* This function will read the connect.dat file.
*
*/
function ReadConnectionStats() {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['data'] . '/connect.dat';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    return $fileString;                     // Return the file string.
}


/**
* Va-Stats.inc.php :: WriteConnectionStats()
*
* This function will write a new connect.dat file based on the file string passed
* to it. If the file string is not in the correct format, then your connect.dat
* file will become corrupt and the BBS could stop functioning!
*
*/
function WriteConnectionStats($fileString) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['data'] . '/connect.dat';
    if (!is_file($file)) return FALSE;

    $status = WriteCfgFile($file, $fileString);

    return $status;                         // Return the result status.
}

?>
